# !/usr/bin/env/python
# -*- coding:utf-8 -*-


""" 
 Auth:  david <hsioe_david@aliyun.com>
 Date:  2020/11/03

 Desc
    HsioeFlask 日志类 
"""
import logging
import os
import time
import codecs
from logging.handlers import BaseRotatingHandler
from hsioe.core.tools import time_tools


class MidnightRotatingFileHandler(BaseRotatingHandler):
    def __init__(self, filename):
        self.suffix = "%Y-%m-%d-%H"
        self.date = time_tools.time_to_str(int(time.time()), self.suffix)
        super(MidnightRotatingFileHandler, self).__init__(
            filename, mode='a', encoding=None, delay=0)

    def shouldRollover(self, record):
        return self.date != time_tools.time_to_str(int(time.time()), self.suffix)

    def doRollover(self):
        if self.stream:
            self.stream.close()
            self.stream = None
        self.date = time_tools.time_to_str(int(time.time()), self.suffix)

    def _open(self):
        filename = '%s.%s' % (self.baseFilename, self.date)
        if self.encoding is None:
            stream = open(filename, self.mode)
        else:
            stream = codecs.open(filename, self.mode, self.encoding)
        if os.path.exists(self.baseFilename):
            try:
                os.remove(self.baseFilename)
            except OSError:
                pass
        try:
            os.symlink(filename, self.baseFilename)
        except OSError:
            pass
        return stream


class HsioeLogger(object):
    """
    日志类
    """

    #: 日志文件目录在LOG
    base_dir = 'logs'

    def __init__(self, name, log_path):
        #: 日志名称
        self._name = name
        self._log_path = log_path
        self.__init()

    def __init(self):
        """
        初始化服务日志
        """
        log_dir_path = self.get_log_dir()
        log_name = "{}/{}.log".format(log_dir_path, self._name)
        self.log_handler = MidnightRotatingFileHandler(log_name)
        self.log_handler.setFormatter(self.formatter())

    def get_log_dir(self):
        """
        获取日志文件目录
        """
        _log_path = os.path.dirname(self._log_path)
        log_dir_folder = os.path.join(_log_path, self.base_dir)

        log_dir_base_folder = log_dir_folder.strip()
        log_dir_log_folder = os.path.join(log_dir_base_folder, self._name)
        if not os.path.exists(log_dir_log_folder):
            os.makedirs(log_dir_log_folder)

        return log_dir_log_folder

    def formatter(self):
        """
        日志格式化
        """
        return logging.Formatter(
            '%(asctime)s - %(levelname)s - %(filename)s - %(funcName)s - %(lineno)s行 - %(message)s'
        )
    
    @classmethod
    def get_logger(cls, log_name, log_path, level) -> logging:
        """
        获取Logger
        """
        log = logging.getLogger(log_name)
        log_handler = cls(log_name, log_path).log_handler
        log.addHandler(log_handler)
        log.setLevel(level)
        return log

    def __repr__(self):
        return "<HsioeLogger:{}>".format(self.__class__.__name__)



# !/usr/bin/env/python
# -*- coding:utf-8 -*-


""" 
 Auth:  david <hsioe_david@aliyun.com>
 Date:  2020/11/10


 Desc
    HsioeFlask Exception测试类
"""
from hsioe.core.exception import HsioeException


class TestHsioeException(object):
    """
    异常测试类
    """

    def test_exception(self, client, hsioe_flask):
        """
        测试函数
        """

        @hsioe_flask.modules.server.route('/10403')
        def raise_10043_exception():
            """
            抛出异常
            JWT_EXPIRED (10403, 'JWT is out of date!')
            HsioeException(exception='JWT_EXPIRED')
            """
            raise HsioeException(error_code=10403, error_msg='hello, world')

        rv = client.get('/10403')
        assert rv.json.get('status') == 10403



# !/usr/bin/env/python
# -*- coding:utf-8 -*-


""" 
 Auth:  david <hsioe_david@aliyun.com>
 Date:  2020/11/06

 Desc
    Hsioe框架测试类配置
"""


from hsioe.app import HsioeFlask
import pytest


class TestConfig:

    secret_key = 'pytest'


@pytest.fixture(scope='session')
def hsioe_flask():

    hsioe_flask = HsioeFlask()
    hsioe_flask.init(config=TestConfig)

    modules = hsioe_flask.modules

    srv_ctx = modules.server.app_context()
    srv_ctx.push()

    return hsioe_flask


@pytest.fixture(scope='session')
def client(hsioe_flask):
    return hsioe_flask.modules.server.test_client()



# !/usr/bin/env/python
# -*- coding:utf-8 -*-


""" 
 Auth:  david <hsioe_david@aliyun.com>
 Date:  


 Desc
    
"""
import datetime


def time_to_str(time_stamp: int, format_str=None):
    """
    时间戳转字符串
    """
    if format_str is None:
        format_str = '%Y-%m-%d %H:%M:%S'

    time_str = datetime.datetime.fromtimestamp(time_stamp)
    return time_str.strftime(format_str)



# !/usr/bin/env/python
# -*- coding:utf-8 -*-


""" 
 Auth:  david <hsioe_david@aliyun.com>
 Date:  2020/11/09


 Desc
    HsioeReponse测试类
"""


class TestHsioeResponse(object):

    def test_response(self, client, hsioe_flask):
        """
        返回验证
        """
        
        @hsioe_flask.modules.server.route('/success')
        def to_success():
            return hsioe_flask.response().success()

        @hsioe_flask.modules.server.route('/fail')
        def to_fail():
            return hsioe_flask.response().fail(fail_tag='JWT_EXPIRED')

        #: 请求成功Response
        rv = client.get('/success')
        assert 'status' in rv.json

        #: 请求失败Response
        rv = client.get('/fail')
        assert 10403 == rv.json.get('status')



# !/usr/bin/env/python
# -*- coding:utf-8 -*-


""" 
 Auth:  david <hsioe_david@aliyun.com>
 Date:  2020/11/09


 Desc
    Hsioe核心类
"""

from .core.logger import HsioeLogger
from .core.event import HsioeEvent
from .core.response import HsioeResponse
from .core.exception import HsioeException
from typing import Tuple, List
import flask
import logging


class HsioeModules(object):
    """
    HsioeModules动态模块类
    """

    def __init__(self):
        object.__setattr__(self, 'modules', {})

    def __getattr__(self, name):

        if name in self.modules:
            return self.modules[name]

        return None

    def __setattr__(self, name, value):

        self.modules[name] = value


class HsioeFlask(object):
    """
    HsioeFlask核心类
    """

    #: HsioeFlaskResponse类型
    response = HsioeResponse

    #: HsioeException异常处理类
    exception = HsioeException

    #: HsioeLogger日志
    logger = HsioeLogger
    
    def __init__(self):
        #: 动态模块
        self.__modules = HsioeModules()

    @property
    def modules(self):
        return self.__modules

    def init(self, config):
        """
        HsioeFlask - Server
        基于Flask对象创建
        """
        self.__modules.server = flask.Flask('Hsioe')
        #: 读取配置文件
        self.__modules.server.config.from_object(config)

        # 装载事件
        self.load_event()

    def load_event(self):
        """
        事件服务
        """
        self.__modules.event = HsioeEvent
    
    def load_plugins(self):
        """
        插件服务
        """
        pass
    
    def load_services(self, services: List[Tuple]) -> None:
        """
        读取子Service
        """
        for server in services:
            try:
                sub_server, url_prefix = server
                self.__modules.server.register_blueprint(sub_server, url_prefix=url_prefix)
            except Exception as e:
                self.server.logger.exception('[LoadServices] exception: {}'.format(e))
                continue
    
    def __repr__(self):
        return "<HsioeFlask Core>"



# !/usr/bin/env/python
# -*- coding:utf-8 -*-


""" 
 Auth:  david <hsioe_david@aliyun.com>
 Date:  2020/11/03  

 Desc
     HsioeFlask Logger测试类
"""
import os
from hsioe.core.logger import HsioeLogger


class TestLogger(object):

    def test_logger(self, hsioe_flask):
        """
        日志模块测试
        """
        open_log = HsioeLogger.get_logger(
            log_name='open', 
            log_path=os.path.dirname(os.path.abspath(__file__)), 
            level='INFO'
        )

        assert open_log is not None



# !/usr/bin/env/python
# -*- coding:utf-8 -*-


""" 
 Auth:  david <hsioe_david@aliyun.com>
 Date:  2020/11/09


 Desc
    HsioeFlask框架Response封装处理
"""
from flask import jsonify

#: Hsioe错误原因映射
#: key: 错误标识KEY
#: value: (状态码, 错误Message)
#: 为了清楚标识错误来源
#: Hsioe框架使用的错误码范围 10000 - 10009
#: version: 0.0.1 
#: david_hsioe@aliyun.com
HSIOE_RESPONSE_FLAG_MAP = {
    'REQUEST_ERROR': (-1000, '请求失败!'),
    'JWT_EXPIRED': (10403, 'Token已过期!'),
    'PARAMS_ERROR': (10001, '请求参数错误!'),
    'PERIMISSION_DENY': (10006, '权限校验失败!')
}


class HsioeResponse(object):
    """
    接口返回类
    
    eg:
        return HsioeResponse().success()
    """

    #: 业务状态默认返回码
    business_code = 200

    #: 业务默认返回消息
    business_message = '请求成功'
    
    #: 业务默认返回data
    business_data = {}

    #: 系统业务的返回标识TAG
    business_tag_map = HSIOE_RESPONSE_FLAG_MAP

    def __init__(self, code=None, message=None, data=None):
        """
        初始化
        """
        self.business_code = code or self.business_code
        self.business_message = message or self.business_message
        self.business_data = data or self.business_data
    
    @property
    def response_data(self):
        return {
            'status': self.business_code,
            'message': self.business_message,
            'data': self.business_data
        }
    
    def do_response(self):
        """
        返回数据
        """
        return jsonify(self.response_data)
    
    def success(self):
        """
        成功请求
        """
        return self.do_response()

    def fail(self, fail_tag):
        """
        失败请求
        """
        if fail_tag not in self.business_tag_map:
            fail_tag = 'REQUEST_ERROR'
        
        self.business_code, self.business_message = self.business_tag_map[fail_tag]
        return self.do_response()

    def __repr__(self):
        return "<HsioeResponse: {} - {}>".format(self.__class__.__name__, self.do_response())


# !/usr/bin/env/python
# -*- coding:utf-8 -*-


""" 
 Auth:  david <hsioe_david@aliyun.com>
 Date:  


 Desc
    
"""
from hsioe import HsioeFlask
from flask import jsonify


class TestEvent(object):
    """
    HsioeFlask Event测试类
    """

    def test_event(self, client, hsioe_flask: HsioeFlask):
        """
        事件测试
        """
        class User:
            score = 1

        add_score_event = hsioe_flask.modules.event(name='user_login', description='hello')

        @add_score_event.register
        def add_score():
            User.score += 1

        @hsioe_flask.modules.server.route('/add_score')
        def to_add_score():
            """
            hello视图
            """
            add_score_event.notify()
            return jsonify(score=User.score)

        #: 判断是否监听到通知
        rv = client.get('/add_score')
        assert rv.json.get('score') == 2



# !/usr/bin/env/python
# -*- coding:utf-8 -*-


""" 
 Auth:  david <hsioe_david@aliyun.com>
 Date:  2020/11/05

 Desc
     HsioeFlask Event事件类
"""


class HsioeEvent(object):
    """
    事件类
    """
    def __init__(self, name, description):

        self._name = name
        self._description = description
        #: 事件列表
        self._event_list = []
    
    def register(self, func):
        """
        事件注册
            #: 此处注册了一个
            login_event = HsioeEvent(name='user_login', description='cessh')

        事件订阅
            @login_event.register
             def hello_test():
               pass
            通过event.register订阅的函数，会收到事件通知的回调
        """
        self._event_list.append(func)

    @property 
    def listeners(self):
        """
        获取所有监听函数
        """
        return self._event_list

    def remove(self, func):
        """
        移除事件
        """
        if func in self._event_list:
            self._event_list.remove(func)
    
    def notify(self, *args, **kwargs):
        """
        通知所有订阅事件
        """
        for event in self._event_list:
            event(*args, **kwargs)
    
    def __repr__(self):
        return "<Event({})>".format(self._name)
